var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var importPath_exports = {};
__export(importPath_exports, {
  default: () => importPath_default,
  getImportFileDistPath: () => getImportFileDistPath,
  getReplacePath: () => getReplacePath,
  isNotJsLikeFile: () => isNotJsLikeFile,
  isShouldSkip: () => isShouldSkip,
  isStyleFile: () => isStyleFile
});
module.exports = __toCommonJS(importPath_exports);
var import_path = __toESM(require("path"));
var import_utils = require("@modern-js/utils");
var import_core = require("@babel/core");
let replaceValueHash = {};
const getImportFileDistPath = (importName, currentFilename, opts) => {
  var _a, _b;
  const { staticDir = ".", styleDir = "." } = opts;
  const currentFileDir = import_path.default.dirname(currentFilename);
  const crtFileRELSrcPath = import_path.default.relative(
    currentFileDir,
    (_a = opts.sourceDir) != null ? _a : currentFileDir
  );
  const importFileRELSrcPath = import_path.default.relative(
    (_b = opts.sourceDir) != null ? _b : currentFileDir,
    import_path.default.join(currentFileDir, importName)
  );
  const resourceDir = isStyleFile(importName) ? styleDir : staticDir;
  const ret = import_path.default.join(crtFileRELSrcPath, resourceDir, importFileRELSrcPath);
  return ret.startsWith(".") ? (0, import_utils.slash)(ret) : (0, import_utils.slash)(`./${ret}`);
};
const isNotJsLikeFile = (importFileName, currentCompileFile) => {
  const tests = [
    [/\.js$/, ".js"],
    [/\.jsx$/, ".jsx"],
    [/\.ts$/, ".ts"],
    [/\.tsx$/, ".tsx"]
  ];
  return !tests.some(
    ([regex, prefix]) => regex.test(importFileName) || import_utils.fs.existsSync(
      import_path.default.join(import_path.default.dirname(currentCompileFile), importFileName) + prefix
    ) || import_utils.fs.existsSync(
      import_path.default.join(import_path.default.dirname(currentCompileFile), importFileName, "index") + prefix
    )
  );
};
const isStyleFile = (file) => {
  const tests = [/\.css$/, /\.less$/, /\.sass$/, /\.scss$/];
  return tests.some((regex) => regex.test(file));
};
const isShouldSkip = (importName, filename) => {
  if (importName.startsWith("./") || importName.startsWith("../")) {
    if (isNotJsLikeFile(importName, filename)) {
      return false;
    }
  }
  return true;
};
const getReplacePath = (opts, importName, currentFilename) => {
  const { importStyle } = opts;
  if (!currentFilename || !importName) {
    return "";
  }
  if (isShouldSkip(importName, currentFilename)) {
    return "";
  }
  let realFilepath = getImportFileDistPath(importName, currentFilename, opts);
  if (isStyleFile(realFilepath) && importStyle === "compiled-code") {
    realFilepath = realFilepath.replace(/\.(less|sass|scss)$/, ".css");
  }
  return realFilepath;
};
const importPath = () => ({
  name: "import-path",
  pre() {
    replaceValueHash = {};
  },
  visitor: {
    Program(nodePath, { opts, file }) {
      nodePath.traverse({
        ImportDeclaration({ node }) {
          const { source } = node;
          const {
            opts: { filename }
          } = file;
          const importName = (source == null ? void 0 : source.value) ? source.value : void 0;
          const replaceValue = getReplacePath(
            opts,
            importName,
            filename
          );
          const hashKey = filename + (importName || "");
          if (replaceValue) {
            if (typeof filename === "string" && !replaceValueHash[hashKey]) {
              node.source.value = replaceValue;
              replaceValueHash[hashKey] = replaceValue;
            } else if (typeof filename === "string" && replaceValueHash[hashKey]) {
              node.source.value = replaceValueHash[hashKey];
            } else {
              node.source.value = replaceValue;
            }
          }
        },
        // dynamic import
        CallExpression({ node }) {
          const { filename } = file.opts;
          const { callee, arguments: args } = node;
          if (callee.type === "Import" || callee.type === "Identifier" && callee.name === "require") {
            const firstArg = args[0];
            if (firstArg.value) {
              const importName = firstArg.value;
              const replaceValue = getReplacePath(
                opts,
                importName,
                filename
              );
              const hashKey = filename + (importName || "");
              if (replaceValue) {
                if (typeof filename === "string" && !replaceValueHash[hashKey]) {
                  node.arguments = [import_core.types.stringLiteral(replaceValue)];
                  replaceValueHash[hashKey] = replaceValue;
                } else if (typeof filename === "string" && replaceValueHash[hashKey]) {
                  node.arguments = [import_core.types.stringLiteral(replaceValueHash[hashKey])];
                } else {
                  node.arguments = [import_core.types.stringLiteral(replaceValue)];
                }
              }
            }
          }
        }
      });
    }
  }
});
var importPath_default = importPath;
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  getImportFileDistPath,
  getReplacePath,
  isNotJsLikeFile,
  isShouldSkip,
  isStyleFile
});
