import path from "path";
import { fs, slash } from "@modern-js/utils";
import { types as t } from "@babel/core";
let replaceValueHash = {};
const getImportFileDistPath = (importName, currentFilename, opts) => {
  var _a, _b;
  const { staticDir = ".", styleDir = "." } = opts;
  const currentFileDir = path.dirname(currentFilename);
  const crtFileRELSrcPath = path.relative(
    currentFileDir,
    (_a = opts.sourceDir) != null ? _a : currentFileDir
  );
  const importFileRELSrcPath = path.relative(
    (_b = opts.sourceDir) != null ? _b : currentFileDir,
    path.join(currentFileDir, importName)
  );
  const resourceDir = isStyleFile(importName) ? styleDir : staticDir;
  const ret = path.join(crtFileRELSrcPath, resourceDir, importFileRELSrcPath);
  return ret.startsWith(".") ? slash(ret) : slash(`./${ret}`);
};
const isNotJsLikeFile = (importFileName, currentCompileFile) => {
  const tests = [
    [/\.js$/, ".js"],
    [/\.jsx$/, ".jsx"],
    [/\.ts$/, ".ts"],
    [/\.tsx$/, ".tsx"]
  ];
  return !tests.some(
    ([regex, prefix]) => regex.test(importFileName) || fs.existsSync(
      path.join(path.dirname(currentCompileFile), importFileName) + prefix
    ) || fs.existsSync(
      path.join(path.dirname(currentCompileFile), importFileName, "index") + prefix
    )
  );
};
const isStyleFile = (file) => {
  const tests = [/\.css$/, /\.less$/, /\.sass$/, /\.scss$/];
  return tests.some((regex) => regex.test(file));
};
const isShouldSkip = (importName, filename) => {
  if (importName.startsWith("./") || importName.startsWith("../")) {
    if (isNotJsLikeFile(importName, filename)) {
      return false;
    }
  }
  return true;
};
const getReplacePath = (opts, importName, currentFilename) => {
  const { importStyle } = opts;
  if (!currentFilename || !importName) {
    return "";
  }
  if (isShouldSkip(importName, currentFilename)) {
    return "";
  }
  let realFilepath = getImportFileDistPath(importName, currentFilename, opts);
  if (isStyleFile(realFilepath) && importStyle === "compiled-code") {
    realFilepath = realFilepath.replace(/\.(less|sass|scss)$/, ".css");
  }
  return realFilepath;
};
const importPath = () => ({
  name: "import-path",
  pre() {
    replaceValueHash = {};
  },
  visitor: {
    Program(nodePath, { opts, file }) {
      nodePath.traverse({
        ImportDeclaration({ node }) {
          const { source } = node;
          const {
            opts: { filename }
          } = file;
          const importName = (source == null ? void 0 : source.value) ? source.value : void 0;
          const replaceValue = getReplacePath(
            opts,
            importName,
            filename
          );
          const hashKey = filename + (importName || "");
          if (replaceValue) {
            if (typeof filename === "string" && !replaceValueHash[hashKey]) {
              node.source.value = replaceValue;
              replaceValueHash[hashKey] = replaceValue;
            } else if (typeof filename === "string" && replaceValueHash[hashKey]) {
              node.source.value = replaceValueHash[hashKey];
            } else {
              node.source.value = replaceValue;
            }
          }
        },
        // dynamic import
        CallExpression({ node }) {
          const { filename } = file.opts;
          const { callee, arguments: args } = node;
          if (callee.type === "Import" || callee.type === "Identifier" && callee.name === "require") {
            const firstArg = args[0];
            if (firstArg.value) {
              const importName = firstArg.value;
              const replaceValue = getReplacePath(
                opts,
                importName,
                filename
              );
              const hashKey = filename + (importName || "");
              if (replaceValue) {
                if (typeof filename === "string" && !replaceValueHash[hashKey]) {
                  node.arguments = [t.stringLiteral(replaceValue)];
                  replaceValueHash[hashKey] = replaceValue;
                } else if (typeof filename === "string" && replaceValueHash[hashKey]) {
                  node.arguments = [t.stringLiteral(replaceValueHash[hashKey])];
                } else {
                  node.arguments = [t.stringLiteral(replaceValue)];
                }
              }
            }
          }
        }
      });
    }
  }
});
var importPath_default = importPath;
export {
  importPath_default as default,
  getImportFileDistPath,
  getReplacePath,
  isNotJsLikeFile,
  isShouldSkip,
  isStyleFile
};
